import { InjectionToken, Component, Inject, Optional, Input, Directive, ViewContainerRef, TemplateRef, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { Subject, ReplaySubject, asapScheduler, of, combineLatest, BehaviorSubject } from 'rxjs';
import { first, takeUntil, distinctUntilChanged } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const HG_ASYNC_RENDER_RESOLVER = new InjectionToken('HG_ASYNC_RENDER_RESOLVER');
/** @enum {number} */
const ResolverConfig = {
    Default: 0,
    AutoResolveOnce: 1,
    AutoResolve: 2,
};
ResolverConfig[ResolverConfig.Default] = 'Default';
ResolverConfig[ResolverConfig.AutoResolveOnce] = 'AutoResolveOnce';
ResolverConfig[ResolverConfig.AutoResolve] = 'AutoResolve';
/**
 * @record
 * @template D
 */
function IActionsTarget() { }
if (false) {
    /** @type {?} */
    IActionsTarget.prototype.loadAction;
    /** @type {?} */
    IActionsTarget.prototype.cancelAction;
    /** @type {?} */
    IActionsTarget.prototype.success$;
    /** @type {?} */
    IActionsTarget.prototype.failure$;
}
/**
 * @template T, D
 */
class AsyncRenderResolver {
    /**
     * @param {?} targetFn
     * @param {?=} dependencies
     */
    constructor(targetFn, dependencies = null) {
        this.targetFn = targetFn;
        this.dependencies = dependencies;
        this.config = ResolverConfig.Default;
        // tslint:disable-next-line:variable-name
        this._isAlive$ = new Subject();
        // tslint:disable-next-line:variable-name
        this._shouldSkip = null;
        // tslint:disable-next-line:variable-name
        this._autoResolveOnceCompleted = false;
        // tslint:disable-next-line:variable-name
        this._resolveRequested = false;
        // tslint:disable-next-line:variable-name
        this._state = { loading: false, errored: false };
        // tslint:disable-next-line:variable-name
        this._data$ = new ReplaySubject(1);
        // tslint:disable-next-line:variable-name
        this._dataObservable$ = this._data$.asObservable();
    }
    /**
     * @return {?}
     */
    get isLoading() { return this._state.loading; }
    /**
     * @return {?}
     */
    get hasErrored() { return this._state.errored; }
    /**
     * @return {?}
     */
    get data$() {
        if (this.isFunctionObservableTarget) {
            return this._dataObservable$;
        }
        // tslint:disable-next-line:max-line-length
        console.warn('hg-async-render: Action based async render resolvers don\'t have data$ property! Data management should be controlled via action handlers!');
        return undefined;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set shouldSkip(value) {
        if (value === true && !this._shouldSkip) {
            if (!!this._functionObservableSubscription) {
                this._functionObservableSubscription.unsubscribe();
                this._functionObservableSubscription = undefined;
            }
            else if (!!this._dependencySubscription) {
                this._dependencySubscription.unsubscribe();
                this._dependencySubscription = undefined;
            }
            this._resolveRequested = false;
            this._state.errored = false;
            this._state.loading = false;
        }
        /** @type {?} */
        const shouldAutoResolveOnce = (this.config === ResolverConfig.AutoResolveOnce && this._autoResolveOnceCompleted === false);
        if (((this.config === ResolverConfig.AutoResolve && this._shouldSkip === true) ||
            (shouldAutoResolveOnce && this._shouldSkip)) && value === false) {
            // asapScheduler.schedule(() => { this.resolve(true); });
            this.resolve(true);
        }
        this._shouldSkip = value;
    }
    /**
     * @return {?}
     */
    get shouldSkip() {
        return this._shouldSkip;
    }
    /**
     * @private
     * @return {?}
     */
    get isFunctionObservableTarget() {
        return this.targetFn instanceof Function;
    }
    /**
     * @param {?=} auto
     * @return {?}
     */
    resolve(auto = false) {
        if (this._resolveRequested || (auto && this._autoResolveOnceCompleted)) {
            return;
        }
        this._resolveRequested = true;
        if (this._dependencySubscription) {
            this._dependencySubscription.unsubscribe();
        }
        /** @type {?} */
        const isAutoResolveOnceConfig = this.config === ResolverConfig.AutoResolveOnce;
        /** @type {?} */
        const isDefaultConfig = this.config === ResolverConfig.Default;
        if (isAutoResolveOnceConfig) {
            this._autoResolveOnceCompleted = true;
        }
        asapScheduler.schedule((/**
         * @return {?}
         */
        () => {
            this._state.errored = false;
            this._state.loading = true;
            this.error = undefined;
            /** @type {?} */
            let dependencies = this.dependencies;
            if (typeof this.dependencies === 'function') {
                dependencies = this.dependencies();
                if (Array.isArray(dependencies)) {
                    dependencies = [dependencies];
                }
            }
            /** @type {?} */
            const deps = !dependencies ? of(undefined) : combineLatest(dependencies).pipe((isAutoResolveOnceConfig || isDefaultConfig) ? first() : takeUntil(this._isAlive$));
            this._dependencySubscription = deps.subscribe((/**
             * @param {?} data
             * @return {?}
             */
            data => {
                this._resolveRequested = false;
                asapScheduler.schedule((/**
                 * @return {?}
                 */
                () => {
                    this._state.errored = false;
                    this._state.loading = true;
                }));
                if (!this.isFunctionObservableTarget) {
                    /** @type {?} */
                    const target = (/** @type {?} */ (this.targetFn));
                    target.loadAction(data);
                    target.success$.pipe(first(), takeUntil(this._isAlive$)).subscribe((/**
                     * @return {?}
                     */
                    () => {
                        this._state.loading = false;
                        this._state.errored = false;
                    }));
                    target.failure$.pipe(first(), takeUntil(this._isAlive$)).subscribe((/**
                     * @return {?}
                     */
                    () => {
                        this._state.loading = false;
                        this._state.errored = true;
                    }));
                }
                else {
                    /** @type {?} */
                    const targetFn = (/** @type {?} */ (this.targetFn));
                    if (this._functionObservableSubscription) {
                        this._functionObservableSubscription.unsubscribe();
                        this._functionObservableSubscription = undefined;
                    }
                    this._functionObservableSubscription = targetFn(data).pipe(takeUntil(this._isAlive$)).subscribe({
                        next: (/**
                         * @param {?} res
                         * @return {?}
                         */
                        res => {
                            this._data$.next(res);
                            this._state.loading = false;
                            this._state.errored = false;
                        }),
                        error: (/**
                         * @param {?} err
                         * @return {?}
                         */
                        err => {
                            this.error = err;
                            this._state.loading = false;
                            this._state.errored = true;
                        }),
                        complete: (/**
                         * @return {?}
                         */
                        () => {
                            this._state.loading = false;
                            this._state.errored = false;
                            this._functionObservableSubscription = undefined;
                        })
                    });
                }
            }));
        }));
    }
    /**
     * @return {?}
     */
    destroy() {
        this._isAlive$.next();
        this._isAlive$.complete();
        if (!this._state.loading) {
            return;
        }
        /** @type {?} */
        const target = (/** @type {?} */ (this.targetFn));
        if (target.cancelAction) {
            target.cancelAction();
        }
    }
}
if (false) {
    /**
     * @type {?}
     * @protected
     */
    AsyncRenderResolver.prototype.config;
    /**
     * @type {?}
     * @private
     */
    AsyncRenderResolver.prototype._isAlive$;
    /**
     * @type {?}
     * @private
     */
    AsyncRenderResolver.prototype._shouldSkip;
    /**
     * @type {?}
     * @private
     */
    AsyncRenderResolver.prototype._autoResolveOnceCompleted;
    /**
     * @type {?}
     * @private
     */
    AsyncRenderResolver.prototype._dependencySubscription;
    /**
     * @type {?}
     * @private
     */
    AsyncRenderResolver.prototype._resolveRequested;
    /**
     * @type {?}
     * @private
     */
    AsyncRenderResolver.prototype._state;
    /**
     * @type {?}
     * @private
     */
    AsyncRenderResolver.prototype._data$;
    /**
     * @type {?}
     * @private
     */
    AsyncRenderResolver.prototype._dataObservable$;
    /**
     * @type {?}
     * @private
     */
    AsyncRenderResolver.prototype._functionObservableSubscription;
    /** @type {?} */
    AsyncRenderResolver.prototype.error;
    /**
     * @type {?}
     * @private
     */
    AsyncRenderResolver.prototype.targetFn;
    /**
     * @type {?}
     * @private
     */
    AsyncRenderResolver.prototype.dependencies;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AsyncRenderBase {
    /**
     * @param {?=} resolvers
     */
    constructor(resolvers = []) {
        this.resolvers = resolvers;
        this.resolvers = [].concat(this.resolvers || []);
    }
    /**
     * @return {?}
     */
    get isLoading() {
        return this.resolvers.reduce((/**
         * @param {?} acc
         * @param {?} res
         * @return {?}
         */
        (acc, res) => acc || res.isLoading), false);
    }
    /**
     * @return {?}
     */
    get hasError() {
        return this.resolvers.reduce((/**
         * @param {?} acc
         * @param {?} res
         * @return {?}
         */
        (acc, res) => acc || res.hasErrored), false);
    }
    /**
     * @protected
     * @return {?}
     */
    destroy() {
        this.resolvers.forEach((/**
         * @param {?} res
         * @return {?}
         */
        res => res.destroy()));
    }
    /**
     * @return {?}
     */
    resolve() {
        this.resolvers.forEach((/**
         * @param {?} res
         * @return {?}
         */
        res => {
            if (res.shouldSkip) {
                return;
            }
            res.resolve();
        }));
    }
}
if (false) {
    /**
     * @type {?}
     * @protected
     */
    AsyncRenderBase.prototype.resolvers;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AsyncRenderComponent extends AsyncRenderBase {
    /**
     * @param {?=} resolvers
     */
    constructor(resolvers = []) {
        super(resolvers);
        this.refresh$ = new Subject();
        this.autoControlLoader = false;
        this.autoControlError = false;
        this.refresh$.subscribe((/**
         * @return {?}
         */
        () => { this.resolve(); }));
    }
    /**
     * @return {?}
     */
    get errors() {
        return this.resolvers.map((/**
         * @param {?} r
         * @return {?}
         */
        r => r.error));
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.resolve();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.refresh$.complete();
        this.destroy();
    }
}
AsyncRenderComponent.decorators = [
    { type: Component, args: [{
                selector: 'hg-async-render',
                template: "<ng-container *ngIf=\"loaderTemplateRef && (!autoControlLoader || isLoading)\">\n  <ng-container *ngTemplateOutlet=\"loaderTemplateRef; context: { $implicit: isLoading }\"></ng-container>\n</ng-container>\n<ng-container *ngIf=\"errorTemplateRef && (!autoControlError || hasError)\">\n  <ng-container *ngTemplateOutlet=\"errorTemplateRef; context: { $implicit: hasError, errors: errors }\"></ng-container>\n</ng-container>\n<ng-content></ng-content>",
                exportAs: 'asyncRender',
                styles: [":host{position:relative;width:100%;display:inline-block}"]
            }] }
];
/** @nocollapse */
AsyncRenderComponent.ctorParameters = () => [
    { type: Array, decorators: [{ type: Inject, args: [HG_ASYNC_RENDER_RESOLVER,] }, { type: Optional }] }
];
AsyncRenderComponent.propDecorators = {
    loaderTemplateRef: [{ type: Input }],
    errorTemplateRef: [{ type: Input }],
    autoControlLoader: [{ type: Input }],
    autoControlError: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    AsyncRenderComponent.prototype.refresh$;
    /** @type {?} */
    AsyncRenderComponent.prototype.loaderTemplateRef;
    /** @type {?} */
    AsyncRenderComponent.prototype.errorTemplateRef;
    /** @type {?} */
    AsyncRenderComponent.prototype.autoControlLoader;
    /** @type {?} */
    AsyncRenderComponent.prototype.autoControlError;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AsyncRenderDirective extends AsyncRenderBase {
    /**
     * @param {?=} resolvers
     * @param {?=} viewContainerRef
     * @param {?=} templateRef
     */
    constructor(resolvers = [], viewContainerRef, templateRef) {
        super(resolvers);
        this.refresh$ = new Subject();
        this.refresh$.subscribe((/**
         * @return {?}
         */
        () => { this.resolve(); }));
        if (!!templateRef) {
            viewContainerRef.createEmbeddedView(templateRef, { $implicit: this });
        }
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.resolve();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.refresh$.complete();
        this.destroy();
    }
}
AsyncRenderDirective.decorators = [
    { type: Directive, args: [{
                selector: '[hgAsyncRender]',
                exportAs: 'asyncRender'
            },] }
];
/** @nocollapse */
AsyncRenderDirective.ctorParameters = () => [
    { type: Array, decorators: [{ type: Inject, args: [HG_ASYNC_RENDER_RESOLVER,] }, { type: Optional }] },
    { type: ViewContainerRef },
    { type: TemplateRef, decorators: [{ type: Optional }] }
];
if (false) {
    /** @type {?} */
    AsyncRenderDirective.prototype.refresh$;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AsyncRenderModule {
}
AsyncRenderModule.decorators = [
    { type: NgModule, args: [{
                declarations: [AsyncRenderComponent, AsyncRenderDirective],
                imports: [
                    CommonModule
                ],
                exports: [AsyncRenderComponent, AsyncRenderDirective]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} target
 * @param {?} key
 * @return {?}
 */
function toObservable(target, key) {
    /** @type {?} */
    const subject = new BehaviorSubject(target[key]);
    /** @type {?} */
    const observable = subject.asObservable().pipe(distinctUntilChanged());
    Object.defineProperty(target, key, {
        set: (/**
         * @param {?} value
         * @return {?}
         */
        (value) => { subject.next(value); }),
        get: (/**
         * @return {?}
         */
        () => observable)
    });
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { AsyncRenderComponent, AsyncRenderDirective, AsyncRenderModule, AsyncRenderResolver, HG_ASYNC_RENDER_RESOLVER, ResolverConfig, toObservable, AsyncRenderBase as ɵa };
//# sourceMappingURL=hg-async-render.js.map
