import { InjectionToken, Component, Inject, Optional, Input, Directive, ViewContainerRef, TemplateRef, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { __extends } from 'tslib';
import { Subject, ReplaySubject, asapScheduler, of, combineLatest, BehaviorSubject } from 'rxjs';
import { first, takeUntil, distinctUntilChanged } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var HG_ASYNC_RENDER_RESOLVER = new InjectionToken('HG_ASYNC_RENDER_RESOLVER');
/** @enum {number} */
var ResolverConfig = {
    Default: 0,
    AutoResolveOnce: 1,
    AutoResolve: 2,
};
ResolverConfig[ResolverConfig.Default] = 'Default';
ResolverConfig[ResolverConfig.AutoResolveOnce] = 'AutoResolveOnce';
ResolverConfig[ResolverConfig.AutoResolve] = 'AutoResolve';
/**
 * @record
 * @template D
 */
function IActionsTarget() { }
if (false) {
    /** @type {?} */
    IActionsTarget.prototype.loadAction;
    /** @type {?} */
    IActionsTarget.prototype.cancelAction;
    /** @type {?} */
    IActionsTarget.prototype.success$;
    /** @type {?} */
    IActionsTarget.prototype.failure$;
}
/**
 * @template T, D
 */
var  /**
 * @template T, D
 */
AsyncRenderResolver = /** @class */ (function () {
    function AsyncRenderResolver(targetFn, dependencies) {
        if (dependencies === void 0) { dependencies = null; }
        this.targetFn = targetFn;
        this.dependencies = dependencies;
        this.config = ResolverConfig.Default;
        // tslint:disable-next-line:variable-name
        this._isAlive$ = new Subject();
        // tslint:disable-next-line:variable-name
        this._shouldSkip = null;
        // tslint:disable-next-line:variable-name
        this._autoResolveOnceCompleted = false;
        // tslint:disable-next-line:variable-name
        this._resolveRequested = false;
        // tslint:disable-next-line:variable-name
        this._state = { loading: false, errored: false };
        // tslint:disable-next-line:variable-name
        this._data$ = new ReplaySubject(1);
        // tslint:disable-next-line:variable-name
        this._dataObservable$ = this._data$.asObservable();
    }
    Object.defineProperty(AsyncRenderResolver.prototype, "isLoading", {
        get: /**
         * @return {?}
         */
        function () { return this._state.loading; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AsyncRenderResolver.prototype, "hasErrored", {
        get: /**
         * @return {?}
         */
        function () { return this._state.errored; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AsyncRenderResolver.prototype, "data$", {
        get: /**
         * @return {?}
         */
        function () {
            if (this.isFunctionObservableTarget) {
                return this._dataObservable$;
            }
            // tslint:disable-next-line:max-line-length
            console.warn('hg-async-render: Action based async render resolvers don\'t have data$ property! Data management should be controlled via action handlers!');
            return undefined;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AsyncRenderResolver.prototype, "shouldSkip", {
        get: /**
         * @return {?}
         */
        function () {
            return this._shouldSkip;
        },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) {
            if (value === true && !this._shouldSkip) {
                if (!!this._functionObservableSubscription) {
                    this._functionObservableSubscription.unsubscribe();
                    this._functionObservableSubscription = undefined;
                }
                else if (!!this._dependencySubscription) {
                    this._dependencySubscription.unsubscribe();
                    this._dependencySubscription = undefined;
                }
                this._resolveRequested = false;
                this._state.errored = false;
                this._state.loading = false;
            }
            /** @type {?} */
            var shouldAutoResolveOnce = (this.config === ResolverConfig.AutoResolveOnce && this._autoResolveOnceCompleted === false);
            if (((this.config === ResolverConfig.AutoResolve && this._shouldSkip === true) ||
                (shouldAutoResolveOnce && this._shouldSkip)) && value === false) {
                // asapScheduler.schedule(() => { this.resolve(true); });
                this.resolve(true);
            }
            this._shouldSkip = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AsyncRenderResolver.prototype, "isFunctionObservableTarget", {
        get: /**
         * @private
         * @return {?}
         */
        function () {
            return this.targetFn instanceof Function;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @param {?=} auto
     * @return {?}
     */
    AsyncRenderResolver.prototype.resolve = /**
     * @param {?=} auto
     * @return {?}
     */
    function (auto) {
        var _this = this;
        if (auto === void 0) { auto = false; }
        if (this._resolveRequested || (auto && this._autoResolveOnceCompleted)) {
            return;
        }
        this._resolveRequested = true;
        if (this._dependencySubscription) {
            this._dependencySubscription.unsubscribe();
        }
        /** @type {?} */
        var isAutoResolveOnceConfig = this.config === ResolverConfig.AutoResolveOnce;
        /** @type {?} */
        var isDefaultConfig = this.config === ResolverConfig.Default;
        if (isAutoResolveOnceConfig) {
            this._autoResolveOnceCompleted = true;
        }
        asapScheduler.schedule((/**
         * @return {?}
         */
        function () {
            _this._state.errored = false;
            _this._state.loading = true;
            _this.error = undefined;
            /** @type {?} */
            var dependencies = _this.dependencies;
            if (typeof _this.dependencies === 'function') {
                dependencies = _this.dependencies();
                if (Array.isArray(dependencies)) {
                    dependencies = [dependencies];
                }
            }
            /** @type {?} */
            var deps = !dependencies ? of(undefined) : combineLatest(dependencies).pipe((isAutoResolveOnceConfig || isDefaultConfig) ? first() : takeUntil(_this._isAlive$));
            _this._dependencySubscription = deps.subscribe((/**
             * @param {?} data
             * @return {?}
             */
            function (data) {
                _this._resolveRequested = false;
                asapScheduler.schedule((/**
                 * @return {?}
                 */
                function () {
                    _this._state.errored = false;
                    _this._state.loading = true;
                }));
                if (!_this.isFunctionObservableTarget) {
                    /** @type {?} */
                    var target = (/** @type {?} */ (_this.targetFn));
                    target.loadAction(data);
                    target.success$.pipe(first(), takeUntil(_this._isAlive$)).subscribe((/**
                     * @return {?}
                     */
                    function () {
                        _this._state.loading = false;
                        _this._state.errored = false;
                    }));
                    target.failure$.pipe(first(), takeUntil(_this._isAlive$)).subscribe((/**
                     * @return {?}
                     */
                    function () {
                        _this._state.loading = false;
                        _this._state.errored = true;
                    }));
                }
                else {
                    /** @type {?} */
                    var targetFn = (/** @type {?} */ (_this.targetFn));
                    if (_this._functionObservableSubscription) {
                        _this._functionObservableSubscription.unsubscribe();
                        _this._functionObservableSubscription = undefined;
                    }
                    _this._functionObservableSubscription = targetFn(data).pipe(takeUntil(_this._isAlive$)).subscribe({
                        next: (/**
                         * @param {?} res
                         * @return {?}
                         */
                        function (res) {
                            _this._data$.next(res);
                            _this._state.loading = false;
                            _this._state.errored = false;
                        }),
                        error: (/**
                         * @param {?} err
                         * @return {?}
                         */
                        function (err) {
                            _this.error = err;
                            _this._state.loading = false;
                            _this._state.errored = true;
                        }),
                        complete: (/**
                         * @return {?}
                         */
                        function () {
                            _this._state.loading = false;
                            _this._state.errored = false;
                            _this._functionObservableSubscription = undefined;
                        })
                    });
                }
            }));
        }));
    };
    /**
     * @return {?}
     */
    AsyncRenderResolver.prototype.destroy = /**
     * @return {?}
     */
    function () {
        this._isAlive$.next();
        this._isAlive$.complete();
        if (!this._state.loading) {
            return;
        }
        /** @type {?} */
        var target = (/** @type {?} */ (this.targetFn));
        if (target.cancelAction) {
            target.cancelAction();
        }
    };
    return AsyncRenderResolver;
}());
if (false) {
    /**
     * @type {?}
     * @protected
     */
    AsyncRenderResolver.prototype.config;
    /**
     * @type {?}
     * @private
     */
    AsyncRenderResolver.prototype._isAlive$;
    /**
     * @type {?}
     * @private
     */
    AsyncRenderResolver.prototype._shouldSkip;
    /**
     * @type {?}
     * @private
     */
    AsyncRenderResolver.prototype._autoResolveOnceCompleted;
    /**
     * @type {?}
     * @private
     */
    AsyncRenderResolver.prototype._dependencySubscription;
    /**
     * @type {?}
     * @private
     */
    AsyncRenderResolver.prototype._resolveRequested;
    /**
     * @type {?}
     * @private
     */
    AsyncRenderResolver.prototype._state;
    /**
     * @type {?}
     * @private
     */
    AsyncRenderResolver.prototype._data$;
    /**
     * @type {?}
     * @private
     */
    AsyncRenderResolver.prototype._dataObservable$;
    /**
     * @type {?}
     * @private
     */
    AsyncRenderResolver.prototype._functionObservableSubscription;
    /** @type {?} */
    AsyncRenderResolver.prototype.error;
    /**
     * @type {?}
     * @private
     */
    AsyncRenderResolver.prototype.targetFn;
    /**
     * @type {?}
     * @private
     */
    AsyncRenderResolver.prototype.dependencies;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var AsyncRenderBase = /** @class */ (function () {
    function AsyncRenderBase(resolvers) {
        if (resolvers === void 0) { resolvers = []; }
        this.resolvers = resolvers;
        this.resolvers = [].concat(this.resolvers || []);
    }
    Object.defineProperty(AsyncRenderBase.prototype, "isLoading", {
        get: /**
         * @return {?}
         */
        function () {
            return this.resolvers.reduce((/**
             * @param {?} acc
             * @param {?} res
             * @return {?}
             */
            function (acc, res) { return acc || res.isLoading; }), false);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AsyncRenderBase.prototype, "hasError", {
        get: /**
         * @return {?}
         */
        function () {
            return this.resolvers.reduce((/**
             * @param {?} acc
             * @param {?} res
             * @return {?}
             */
            function (acc, res) { return acc || res.hasErrored; }), false);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @protected
     * @return {?}
     */
    AsyncRenderBase.prototype.destroy = /**
     * @protected
     * @return {?}
     */
    function () {
        this.resolvers.forEach((/**
         * @param {?} res
         * @return {?}
         */
        function (res) { return res.destroy(); }));
    };
    /**
     * @return {?}
     */
    AsyncRenderBase.prototype.resolve = /**
     * @return {?}
     */
    function () {
        this.resolvers.forEach((/**
         * @param {?} res
         * @return {?}
         */
        function (res) {
            if (res.shouldSkip) {
                return;
            }
            res.resolve();
        }));
    };
    return AsyncRenderBase;
}());
if (false) {
    /**
     * @type {?}
     * @protected
     */
    AsyncRenderBase.prototype.resolvers;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var AsyncRenderComponent = /** @class */ (function (_super) {
    __extends(AsyncRenderComponent, _super);
    function AsyncRenderComponent(resolvers) {
        if (resolvers === void 0) { resolvers = []; }
        var _this = _super.call(this, resolvers) || this;
        _this.refresh$ = new Subject();
        _this.autoControlLoader = false;
        _this.autoControlError = false;
        _this.refresh$.subscribe((/**
         * @return {?}
         */
        function () { _this.resolve(); }));
        return _this;
    }
    Object.defineProperty(AsyncRenderComponent.prototype, "errors", {
        get: /**
         * @return {?}
         */
        function () {
            return this.resolvers.map((/**
             * @param {?} r
             * @return {?}
             */
            function (r) { return r.error; }));
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    AsyncRenderComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        this.resolve();
    };
    /**
     * @return {?}
     */
    AsyncRenderComponent.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.refresh$.complete();
        this.destroy();
    };
    AsyncRenderComponent.decorators = [
        { type: Component, args: [{
                    selector: 'hg-async-render',
                    template: "<ng-container *ngIf=\"loaderTemplateRef && (!autoControlLoader || isLoading)\">\n  <ng-container *ngTemplateOutlet=\"loaderTemplateRef; context: { $implicit: isLoading }\"></ng-container>\n</ng-container>\n<ng-container *ngIf=\"errorTemplateRef && (!autoControlError || hasError)\">\n  <ng-container *ngTemplateOutlet=\"errorTemplateRef; context: { $implicit: hasError, errors: errors }\"></ng-container>\n</ng-container>\n<ng-content></ng-content>",
                    exportAs: 'asyncRender',
                    styles: [":host{position:relative;width:100%;display:inline-block}"]
                }] }
    ];
    /** @nocollapse */
    AsyncRenderComponent.ctorParameters = function () { return [
        { type: Array, decorators: [{ type: Inject, args: [HG_ASYNC_RENDER_RESOLVER,] }, { type: Optional }] }
    ]; };
    AsyncRenderComponent.propDecorators = {
        loaderTemplateRef: [{ type: Input }],
        errorTemplateRef: [{ type: Input }],
        autoControlLoader: [{ type: Input }],
        autoControlError: [{ type: Input }]
    };
    return AsyncRenderComponent;
}(AsyncRenderBase));
if (false) {
    /** @type {?} */
    AsyncRenderComponent.prototype.refresh$;
    /** @type {?} */
    AsyncRenderComponent.prototype.loaderTemplateRef;
    /** @type {?} */
    AsyncRenderComponent.prototype.errorTemplateRef;
    /** @type {?} */
    AsyncRenderComponent.prototype.autoControlLoader;
    /** @type {?} */
    AsyncRenderComponent.prototype.autoControlError;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var AsyncRenderDirective = /** @class */ (function (_super) {
    __extends(AsyncRenderDirective, _super);
    function AsyncRenderDirective(resolvers, viewContainerRef, templateRef) {
        if (resolvers === void 0) { resolvers = []; }
        var _this = _super.call(this, resolvers) || this;
        _this.refresh$ = new Subject();
        _this.refresh$.subscribe((/**
         * @return {?}
         */
        function () { _this.resolve(); }));
        if (!!templateRef) {
            viewContainerRef.createEmbeddedView(templateRef, { $implicit: _this });
        }
        return _this;
    }
    /**
     * @return {?}
     */
    AsyncRenderDirective.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        this.resolve();
    };
    /**
     * @return {?}
     */
    AsyncRenderDirective.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.refresh$.complete();
        this.destroy();
    };
    AsyncRenderDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[hgAsyncRender]',
                    exportAs: 'asyncRender'
                },] }
    ];
    /** @nocollapse */
    AsyncRenderDirective.ctorParameters = function () { return [
        { type: Array, decorators: [{ type: Inject, args: [HG_ASYNC_RENDER_RESOLVER,] }, { type: Optional }] },
        { type: ViewContainerRef },
        { type: TemplateRef, decorators: [{ type: Optional }] }
    ]; };
    return AsyncRenderDirective;
}(AsyncRenderBase));
if (false) {
    /** @type {?} */
    AsyncRenderDirective.prototype.refresh$;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var AsyncRenderModule = /** @class */ (function () {
    function AsyncRenderModule() {
    }
    AsyncRenderModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [AsyncRenderComponent, AsyncRenderDirective],
                    imports: [
                        CommonModule
                    ],
                    exports: [AsyncRenderComponent, AsyncRenderDirective]
                },] }
    ];
    return AsyncRenderModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} target
 * @param {?} key
 * @return {?}
 */
function toObservable(target, key) {
    /** @type {?} */
    var subject = new BehaviorSubject(target[key]);
    /** @type {?} */
    var observable = subject.asObservable().pipe(distinctUntilChanged());
    Object.defineProperty(target, key, {
        set: (/**
         * @param {?} value
         * @return {?}
         */
        function (value) { subject.next(value); }),
        get: (/**
         * @return {?}
         */
        function () { return observable; })
    });
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { AsyncRenderComponent, AsyncRenderDirective, AsyncRenderModule, AsyncRenderResolver, HG_ASYNC_RENDER_RESOLVER, ResolverConfig, toObservable, AsyncRenderBase as ɵa };
//# sourceMappingURL=hg-async-render.js.map
